/**
 * SmartCGMS - continuous glucose monitoring and controlling framework
 * https://diabetes.zcu.cz/
 *
 * Copyright (c) since 2018 University of West Bohemia.
 *
 * Contact:
 * diabetes@mail.kiv.zcu.cz
 * Medical Informatics, Department of Computer Science and Engineering
 * Faculty of Applied Sciences, University of West Bohemia
 * Univerzitni 8, 301 00 Pilsen
 * Czech Republic
 * 
 * 
 * Purpose of this software:
 * This software is intended to demonstrate work of the diabetes.zcu.cz research
 * group to other scientists, to complement our published papers. It is strictly
 * prohibited to use this software for diagnosis or treatment of any medical condition,
 * without obtaining all required approvals from respective regulatory bodies.
 *
 * Especially, a diabetic patient is warned that unauthorized use of this software
 * may result into severe injure, including death.
 *
 *
 * Licensing terms:
 * Unless required by applicable law or agreed to in writing, software
 * distributed under these license terms is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *
 * a) This file is available under the Apache License, Version 2.0.
 * b) When publishing any derivative work or results obtained using this software, you agree to cite the following paper:
 *    Tomas Koutny and Martin Ubl, "SmartCGMS as a Testbed for a Blood-Glucose Level Prediction and/or 
 *    Control Challenge with (an FDA-Accepted) Diabetic Patient Simulation", Procedia Computer Science,  
 *    Volume 177, pp. 354-362, 2020
 */

#pragma once

#include <scgms/iface/UIIface.h>

namespace generated123_model {

	// BP NOTE: tohle GUID je treba vzdy vygenerovat
	const GUID model_id = { 0x83332b3a, 0x46a8, 0x4501, { 0xa4, 0xb3, 0x90, 0xa1, 0xc2, 0xac, 0x58, 0xa5 } }; // {83332B3A-46A8-4501-A4B3-90A1C2AC58A5}

	// BP NOTE: pocet parametru generujte podle toho co si uzivatel naklika
	constexpr size_t model_param_count = 5;

	// BP NOTE: tohle nechte, jak je
	struct TParameters {
		double vector[model_param_count];
	};

	// BP NOTE: obsah tohoto pole generujte dle toho, co si uzivatel zvoli - spodni hranice parametru
	const TParameters lower_bounds = {
		0, 0.5, -1, 0, 0
	};

	// BP NOTE: obsah tohoto pole generujte dle toho, co si uzivatel zvoli - vychozi hodnoty parametru
	const TParameters default_parameters = {
		0.2, 0.7, 0.1, 0.05, 0.01
	};

	// BP NOTE: obsah tohoto pole generujte dle toho, co si uzivatel zvoli - horni hranice parametru
	const TParameters upper_bounds = {
		1, 2, 1.45, 0.7, 0.55
	};

	// BP NOTE: tohle GUID pro kazdy vystupni signal vygenerujte take vlastni
	const GUID signal_Output_1 = { 0x84b7004b, 0x84b9, 0x45ec, { 0x91, 0xc3, 0xe, 0x50, 0xdd, 0xa3, 0x4f, 0x54 } }; // {84B7004B-84B9-45EC-91C3-0E50DDA34F54}

	// BP NOTE: tohle GUID musi zadat uzivatel, sem ho jen zkopirujte z GUI - jde o vstupni signal na ktery reaguje model
	const GUID signal_Input_1 = { 0x3034568d, 0xf498, 0x455b,{ 0xac, 0x6a, 0xbc, 0xf3, 0x1, 0xf6, 0x9c, 0x9e } };
}
