/**
 * SmartCGMS - continuous glucose monitoring and controlling framework
 * https://diabetes.zcu.cz/
 *
 * Copyright (c) since 2018 University of West Bohemia.
 *
 * Contact:
 * diabetes@mail.kiv.zcu.cz
 * Medical Informatics, Department of Computer Science and Engineering
 * Faculty of Applied Sciences, University of West Bohemia
 * Univerzitni 8, 301 00 Pilsen
 * Czech Republic
 * 
 * 
 * Purpose of this software:
 * This software is intended to demonstrate work of the diabetes.zcu.cz research
 * group to other scientists, to complement our published papers. It is strictly
 * prohibited to use this software for diagnosis or treatment of any medical condition,
 * without obtaining all required approvals from respective regulatory bodies.
 *
 * Especially, a diabetic patient is warned that unauthorized use of this software
 * may result into severe injure, including death.
 *
 *
 * Licensing terms:
 * Unless required by applicable law or agreed to in writing, software
 * distributed under these license terms is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *
 * a) This file is available under the Apache License, Version 2.0.
 * b) When publishing any derivative work or results obtained using this software, you agree to cite the following paper:
 *    Tomas Koutny and Martin Ubl, "SmartCGMS as a Testbed for a Blood-Glucose Level Prediction and/or 
 *    Control Challenge with (an FDA-Accepted) Diabetic Patient Simulation", Procedia Computer Science,  
 *    Volume 177, pp. 354-362, 2020
 */

#pragma once

#include "descriptor.h"

#include <functional>

#include <scgms/rtl/FilterLib.h>

#include "ode/ode_solvers.h"
#include "ode/ode_solver_parameters.h"

 // BP NOTE: tohle vygenerujte podle toho, kolik model obsahuje kompartmentu
constexpr size_t Num_Compartments = 4;

struct CModel_State
{
	double lastTime;
	double C[Num_Compartments];
};

class CGenerated123_Model;

namespace model_support
{
	// BP NOTE: jen vygenerujte spravne jmeno tridy

	using TDiff_Eq_Fnc = double (CGenerated123_Model::*)(const double, const double) const;
	using TDiff_Eq = decltype(std::bind<double>(std::declval<TDiff_Eq_Fnc>(), std::declval<CGenerated123_Model*>(), std::declval<const decltype(std::placeholders::_1)&>(), std::declval<const decltype(std::placeholders::_2)&>()));

	struct CEquation_Binding
	{
		double& x;
		TDiff_Eq fnc;
	};
}

#pragma warning( push )
#pragma warning( disable : 4250 ) // C4250 - 'class1' : inherits 'class2::member' via dominance

// BP NOTE: jen vygenerujte spravne jmeno modelu
class CGenerated123_Model : public scgms::CBase_Filter, public scgms::IDiscrete_Model
{
	private:
		const double ODE_epsilon0 = 0.001;
		const size_t mODE_Max_Steps = 100;
	private:
		// BP NOTE: jen vygenerujte spravne jmeno namespace
		generated123_model::TParameters mParameters;

		CModel_State mState;
		bool mInitialized = false;
		const std::vector<model_support::CEquation_Binding> mEquation_Binding;
		ode::default_solver ODE_Solver{ ODE_epsilon0, mODE_Max_Steps };

	private:
		// BP NOTE: tohle generujte pro kazdy kompartment dle sablony - bude se menit jen jmeno
		double eq_dC0(const double _T, const double _X) const;
		double eq_dC1(const double _T, const double _X) const;
		double eq_dC2(const double _T, const double _X) const;
		double eq_dC3(const double _T, const double _X) const;

	protected:
		uint64_t mSegment_Id = scgms::Invalid_Segment_Id;
		HRESULT Emit_Signal_Level(const GUID& signal_id, double device_time, double level);
		void Emit_All_Signals(double time_advance_delta);

	protected:
		// scgms::CBase_Filter iface implementation
		virtual HRESULT Do_Execute(scgms::UDevice_Event event) override final;
		virtual HRESULT Do_Configure(scgms::SFilter_Configuration configuration, refcnt::Swstr_list& error_description) override final;

	public:
		// BP NOTE: jen vygenerujte spravne jmeno modelu
		CGenerated123_Model(scgms::IModel_Parameter_Vector *parameters, scgms::IFilter *output);
		virtual ~CGenerated123_Model() = default;

		// scgms::IDiscrete_Model iface
		virtual HRESULT IfaceCalling Initialize(const double current_time, const uint64_t segment_id) override final;
		virtual HRESULT IfaceCalling Step(const double time_advance_delta) override final;
};

#pragma warning( pop )
